@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
+ * Copyright (c) 2019-22, NVIDIA CORPORATION & AFFILIATES.  All rights reserved.
  *
  * NVIDIA CORPORATION and its licensors retain all intellectual property
  * and proprietary rights in and to this software, related documentation
@@ -17,10 +17,12 @@
 #pragma once
 
 #define CUTENSOR_MAJOR 1 //!< cuTensor major version.
-#define CUTENSOR_MINOR 3 //!< cuTensor minor version.
-#define CUTENSOR_PATCH 1 //!< cuTensor patch version.
+#define CUTENSOR_MINOR 5 //!< cuTensor minor version.
+#define CUTENSOR_PATCH 0 //!< cuTensor patch version.
+#define CUTENSOR_VERSION (CUTENSOR_MAJOR * 10000 + CUTENSOR_MINOR * 100 + CUTENSOR_PATCH)
 
 #include <stdint.h>
+#include <stdio.h>
 #include <cuda_runtime.h>
 
 #include <cutensor/types.h>
@@ -252,20 +254,32 @@ cutensorStatus_t cutensorInitTensorDescriptor(const cutensorHandle_t* handle,
  *
  * Supported data-type combinations are:
  *
- * \verbatim
- * |    typeA   |    typeB   |    typeC   | typeScalar |
- * | -----------|------------|------------|-------------|
- * | CUDA_R_16F | CUDA_R_16F | CUDA_R_16F | CUDA_R_16F  |
- * | CUDA_R_16F | CUDA_R_16F | CUDA_R_16F | CUDA_R_32F  |
- * | CUDA_R_16BF| CUDA_R_16BF| CUDA_R_16BF| CUDA_R_16BF |
- * | CUDA_R_16BF| CUDA_R_16BF| CUDA_R_16BF| CUDA_R_32F  |
- * | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F  |
- * | CUDA_R_64F | CUDA_R_64F | CUDA_R_64F | CUDA_R_64F  |
- * | CUDA_C_32F | CUDA_C_32F | CUDA_C_32F | CUDA_C_32F  |
- * | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F  |
- * | CUDA_R_32F | CUDA_R_32F | CUDA_R_16F | CUDA_R_32F  |
- * | CUDA_R_64F | CUDA_R_64F | CUDA_R_32F | CUDA_R_64F  |
- * | CUDA_C_64F | CUDA_C_64F | CUDA_C_32F | CUDA_C_64F  |
+ * \verbatim embed:rst:leading-asterisk
+ * +---------------+---------------+---------------+---------------+
+ * |     typeA     |     typeB     |     typeC     |  typeScalar   |
+ * +===============+===============+===============+===============+
+ * |  CUDA_R_16F   |  CUDA_R_16F   |  CUDA_R_16F   |  CUDA_R_16F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_R_16F   |  CUDA_R_16F   |  CUDA_R_16F   |  CUDA_R_32F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_R_16BF  |  CUDA_R_16BF  |  CUDA_R_16BF  |  CUDA_R_16BF  |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_R_16BF  |  CUDA_R_16BF  |  CUDA_R_16BF  |  CUDA_R_32F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_R_32F   |  CUDA_R_32F   |  CUDA_R_32F   |  CUDA_R_32F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_R_64F   |  CUDA_R_64F   |  CUDA_R_64F   |  CUDA_R_64F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_C_32F   |  CUDA_C_32F   |  CUDA_C_32F   |  CUDA_C_32F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_C_64F   |  CUDA_C_64F   |  CUDA_C_64F   |  CUDA_C_64F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_R_32F   |  CUDA_R_32F   |  CUDA_R_16F   |  CUDA_R_32F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_R_64F   |  CUDA_R_64F   |  CUDA_R_32F   |  CUDA_R_64F   |
+ * +---------------+---------------+---------------+---------------+
+ * |  CUDA_C_64F   |  CUDA_C_64F   |  CUDA_C_32F   |  CUDA_C_64F   |
+ * +---------------+---------------+---------------+---------------+
  * \endverbatim
  *
  * \param[in] handle Opaque handle holding cuTENSOR's library context.
@@ -485,7 +499,7 @@ cutensorStatus_t cutensorContractionFindSetAttribute(
  * \retval CUTENSOR_STATUS_NOT_INITIALIZED if the handle is not initialized.
  * \retval CUTENSOR_STATUS_INVALID_VALUE if some input data is invalid (this typically indicates an user error).
  */
-cutensorStatus_t cutensorContractionGetWorkspace(const cutensorHandle_t* handle,
+cutensorStatus_t cutensorContractionGetWorkspaceSize(const cutensorHandle_t* handle,
                                                  const cutensorContractionDescriptor_t* desc,
                                                  const cutensorContractionFind_t* find,
                                                  const cutensorWorksizePreference_t pref,
@@ -529,22 +543,38 @@ cutensorStatus_t cutensorInitContractionPlan(const cutensorHandle_t* handle,
  *
  * Supported data-type combinations are:
  *
- * \verbatim
- * |    typeA   |    typeB   |    typeC   | typeCompute |
- * | -----------|------------|------------|-------------|
- * | CUDA_R_16F | CUDA_R_16F | CUDA_R_16F | CUDA_R_32F  |
- * | CUDA_R_16BF| CUDA_R_16BF| CUDA_R_16BF| CUDA_R_32F  |
- * | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F  |
- * | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F | CUDA_R_TF32 |
- * | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F | CUDA_R_16F  |
- * | CUDA_R_64F | CUDA_R_64F | CUDA_R_64F | CUDA_R_64F  |
- * | CUDA_R_64F | CUDA_R_64F | CUDA_R_64F | CUDA_R_32F  |
- * | CUDA_C_32F | CUDA_C_32F | CUDA_C_32F | CUDA_C_32F  |
- * | CUDA_C_32F | CUDA_C_32F | CUDA_C_32F | CUDA_C_TF32 |
- * | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F  |
- * | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F | CUDA_C_32F  |
- * | CUDA_R_64F | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F  |
- * | CUDA_C_64F | CUDA_R_64F | CUDA_C_64F | CUDA_C_64F  |
+ * \verbatim embed:rst:leading-asterisk
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |     typeA     |     typeB     |     typeC     |        typeCompute      | Tensor Core |
+ * +===============+===============+===============+=========================+=============+
+ * |  CUDA_R_16F   |  CUDA_R_16F   |  CUDA_R_16F   |  CUTENSOR_COMPUTE_32F   | Volta+      |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_16BF  |  CUDA_R_16BF  |  CUDA_R_16BF  |  CUTENSOR_COMPUTE_32F   | Ampere+     |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_32F   |  CUDA_R_32F   |  CUDA_R_32F   |  CUTENSOR_COMPUTE_32F   | No          |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_32F   |  CUDA_R_32F   |  CUDA_R_32F   |  CUTENSOR_COMPUTE_TF32  | Ampere+     |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_32F   |  CUDA_R_32F   |  CUDA_R_32F   |  CUTENSOR_COMPUTE_16BF  | Ampere+     |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_32F   |  CUDA_R_32F   |  CUDA_R_32F   |  CUTENSOR_COMPUTE_16F   | Volta+      |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_64F   |  CUDA_R_64F   |  CUDA_R_64F   |  CUTENSOR_COMPUTE_64F   | Ampere+     |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_64F   |  CUDA_R_64F   |  CUDA_R_64F   |  CUTENSOR_COMPUTE_32F   | No          |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_C_32F   |  CUDA_C_32F   |  CUDA_C_32F   |  CUTENSOR_COMPUTE_32F   | No          |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_C_32F   |  CUDA_C_32F   |  CUDA_C_32F   |  CUTENSOR_COMPUTE_TF32  | Ampere+     |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_C_64F   |  CUDA_C_64F   |  CUDA_C_64F   |  CUTENSOR_COMPUTE_64F   | Ampere+     |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_C_64F   |  CUDA_C_64F   |  CUDA_C_64F   |  CUTENSOR_COMPUTE_32F   | No          |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_R_64F   |  CUDA_C_64F   |  CUDA_C_64F   |  CUTENSOR_COMPUTE_64F   | No          |
+ * +---------------+---------------+---------------+-------------------------+-------------+
+ * |  CUDA_C_64F   |  CUDA_R_64F   |  CUDA_C_64F   |  CUTENSOR_COMPUTE_64F   | No          |
+ * +---------------+---------------+---------------+-------------------------+-------------+
  * \endverbatim
  *
  * \param[in] handle Opaque handle holding cuTENSOR's library context.
@@ -566,7 +596,9 @@ cutensorStatus_t cutensorInitContractionPlan(const cutensorHandle_t* handle,
  * \retval CUTENSOR_STATUS_INVALID_VALUE if some input data is invalid (this typically indicates an user error).
  * \retval CUTENSOR_STATUS_SUCCESS The operation completed successfully.
  * \retval CUTENSOR_STATUS_NOT_INITIALIZED if the handle is not initialized.
- * \retval CUTENSOR_STATUS_CUDA_ERROR if some unknown CUDA error has occurred.
+ * \retval CUTENSOR_STATUS_ARCH_MISMATCH if the plan was created for a different device than the currently active device.
+ * \retval CUTENSOR_STATUS_INSUFFICIENT_DRIVER if the driver is insufficient.
+ * \retval CUTENSOR_STATUS_CUDA_ERROR if some unknown CUDA error has occurred (e.g., out of memory).
  */
 cutensorStatus_t cutensorContraction(const cutensorHandle_t* handle, 
                                      const cutensorContractionPlan_t* plan,
@@ -598,17 +630,26 @@ cutensorStatus_t cutensorContractionMaxAlgos(int32_t* maxNumAlgos);
  *
  * Supported data-type combinations are:
  *
- * \verbatim
- * |    typeA   |    typeB   |    typeC   | typeCompute |
- * | -----------|------------|------------|-------------|
- * | CUDA_R_16F | CUDA_R_16F | CUDA_R_16F | CUDA_R_16F  |
- * | CUDA_R_16F | CUDA_R_16F | CUDA_R_16F | CUDA_R_32F  |
- * | CUDA_R_16BF| CUDA_R_16BF| CUDA_R_16BF| CUDA_R_16BF |
- * | CUDA_R_16BF| CUDA_R_16BF| CUDA_R_16BF| CUDA_R_32F  |
- * | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F | CUDA_R_32F  |
- * | CUDA_R_64F | CUDA_R_64F | CUDA_R_64F | CUDA_R_64F  |
- * | CUDA_C_32F | CUDA_C_32F | CUDA_C_32F | CUDA_C_32F  |
- * | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F | CUDA_C_64F  |
+ * \verbatim embed:rst:leading-asterisk
+ * +---------------+---------------+---------------+-------------------------+
+ * |     typeA     |     typeB     |     typeC     |       typeCompute       |
+ * +===============+===============+===============+=========================+
+ * | `CUDA_R_16F`  | `CUDA_R_16F`  | `CUDA_R_16F`  | `CUTENSOR_COMPUTE_16F`  |
+ * +---------------+---------------+---------------+-------------------------+
+ * | `CUDA_R_16F`  | `CUDA_R_16F`  | `CUDA_R_16F`  | `CUTENSOR_COMPUTE_32F`  |
+ * +---------------+---------------+---------------+-------------------------+
+ * | `CUDA_R_16BF` | `CUDA_R_16BF` | `CUDA_R_16BF` | `CUTENSOR_COMPUTE_16BF` |
+ * +---------------+---------------+---------------+-------------------------+
+ * | `CUDA_R_16BF` | `CUDA_R_16BF` | `CUDA_R_16BF` | `CUTENSOR_COMPUTE_32F`  |
+ * +---------------+---------------+---------------+-------------------------+
+ * | `CUDA_R_32F`  | `CUDA_R_32F`  | `CUDA_R_32F`  | `CUTENSOR_COMPUTE_32F`  |
+ * +---------------+---------------+---------------+-------------------------+
+ * | `CUDA_R_64F`  | `CUDA_R_64F`  | `CUDA_R_64F`  | `CUTENSOR_COMPUTE_64F`  |
+ * +---------------+---------------+---------------+-------------------------+
+ * | `CUDA_C_32F`  | `CUDA_C_32F`  | `CUDA_C_32F`  | `CUTENSOR_COMPUTE_32F`  |
+ * +---------------+---------------+---------------+-------------------------+
+ * | `CUDA_C_64F`  | `CUDA_C_64F`  | `CUDA_C_64F`  | `CUTENSOR_COMPUTE_64F`  |
+ * +---------------+---------------+---------------+-------------------------+
  * \endverbatim
  *
  * \param[in] handle Opaque handle holding cuTENSOR's library context.
@@ -626,7 +667,7 @@ cutensorStatus_t cutensorContractionMaxAlgos(int32_t* maxNumAlgos);
  * \param[in] opReduce binary operator used to reduce elements of A.
  * \param[in] typeCompute All arithmetic is performed using this data type (i.e., it affects the accuracy and performance).
  * \param[out] workspace Scratchpad (device) memory; the workspace must be aligned to 128 bytes.
- * \param[in] workspaceSize Please use cutensorReductionGetWorkspace() to query the required workspace.
+ * \param[in] workspaceSize Please use cutensorReductionGetWorkspaceSize() to query the required workspace.
  *            While lower values, including zero, are valid, they may lead to grossly suboptimal performance.
  * \param[in] stream The CUDA stream in which all the computation is performed.
  *
@@ -665,7 +706,7 @@ cutensorStatus_t cutensorReduction(const cutensorHandle_t* handle,
  * \retval CUTENSOR_STATUS_NOT_INITIALIZED if the handle is not initialized.
  * \retval CUTENSOR_STATUS_INVALID_VALUE if some input data is invalid (this typically indicates an user error).
  */
-cutensorStatus_t cutensorReductionGetWorkspace(const cutensorHandle_t* handle, 
+cutensorStatus_t cutensorReductionGetWorkspaceSize(const cutensorHandle_t* handle, 
         const void* A, const cutensorTensorDescriptor_t* descA, const int32_t modeA[],
         const void* C, const cutensorTensorDescriptor_t* descC, const int32_t modeC[],
         const void* D, const cutensorTensorDescriptor_t* descD, const int32_t modeD[],
@@ -704,6 +745,105 @@ size_t cutensorGetVersion();
  */
 size_t cutensorGetCudartVersion();
 
+/**
+ * \brief This function sets the logging callback routine.
+ * \param[in] callback Pointer to a callback function. Check cutensorLoggerCallback_t.
+ */
+cutensorStatus_t cutensorLoggerSetCallback(cutensorLoggerCallback_t callback);
+
+/**
+ * \brief This function sets the logging output file.
+ * \param[in] file An open file with write permission.
+ */
+cutensorStatus_t cutensorLoggerSetFile(FILE* file);
+
+/**
+ * \brief This function opens a logging output file in the given path.
+ * \param[in] logFile Path to the logging output file.
+ */
+cutensorStatus_t cutensorLoggerOpenFile(const char* logFile);
+
+/**
+ * \brief This function sets the value of the logging level.
+ * \param[in] level Log level, should be one of the following:
+ *                  0.  Off
+ *                  1.  Errors
+ *                  2.  Performance Trace
+ *                  3.  Performance Hints
+ *                  4.  Heuristics Trace
+ *                  5.  API Trace
+ */
+cutensorStatus_t cutensorLoggerSetLevel(int32_t level);
+
+/**
+ * \brief This function sets the value of the log mask.
+ * \param[in] mask Log mask, the bitwise OR of the following:
+ *                 0.  Off
+ *                 1.  Errors
+ *                 2.  Performance Trace
+ *                 4.  Performance Hints
+ *                 8.  Heuristics Trace
+ *                 16. API Trace
+ *
+ */
+cutensorStatus_t cutensorLoggerSetMask(int32_t mask);
+
+/**
+ * \brief This function disables logging for the entire run.
+ */
+cutensorStatus_t cutensorLoggerForceDisable();
+
+#ifdef _MSC_VER
+#define CUTENSOR_DEPRECATED(new_func) __declspec(deprecated("please use " #new_func " instead."))
+#elif defined(__GNUC__)
+#define CUTENSOR_DEPRECATED(new_func) __attribute__((deprecated("please use " #new_func " instead.")))
+#else
+#define CUTENSOR_DEPRECATED(new_func)
+#endif
+
+/**
+ * \brief DEPRECATED: Determines the required workspaceSize for a given tensor contraction (see \ref cutensorContraction)
+ *
+ * \param[in] handle Opaque handle holding cuTENSOR's library context.
+ * \param[in] desc This opaque struct encodes the tensor contraction problem.
+ * \param[in] find This opaque struct restricts the search space of viable candidates.
+ * \param[in] pref This parameter influences the size of the workspace; see \ref cutensorWorksizePreference_t for details.
+ * \param[out] workspaceSize The workspace size (in bytes) that is required for the given tensor contraction.
+ * \retval CUTENSOR_STATUS_SUCCESS The operation completed successfully.
+ * \retval CUTENSOR_STATUS_NOT_INITIALIZED if the handle is not initialized.
+ * \retval CUTENSOR_STATUS_INVALID_VALUE if some input data is invalid (this typically indicates an user error).
+ */
+cutensorStatus_t CUTENSOR_DEPRECATED(cutensorContractionGetWorkspaceSize) cutensorContractionGetWorkspace(const cutensorHandle_t* handle,
+                                                 const cutensorContractionDescriptor_t* desc,
+                                                 const cutensorContractionFind_t* find,
+                                                 const cutensorWorksizePreference_t pref,
+                                                 uint64_t *workspaceSize);
+
+/**
+ * \brief DEPRECATED: Determines the required workspaceSize for a given tensor reduction (see \ref cutensorReduction)
+ *
+ * \param[in] handle Opaque handle holding cuTENSOR's library context.
+ * \param[in] A same as in cutensorReduction
+ * \param[in] descA same as in cutensorReduction
+ * \param[in] modeA same as in cutensorReduction
+ * \param[in] C same as in cutensorReduction
+ * \param[in] descC same as in cutensorReduction
+ * \param[in] modeC same as in cutensorReduction
+ * \param[in] D same as in cutensorReduction
+ * \param[in] descD same as in cutensorReduction
+ * \param[in] modeD same as in cutensorReduction
+ * \param[in] opReduce same as in cutensorReduction
+ * \param[in] typeCompute same as in cutensorReduction
+ * \param[out] workspaceSize The workspace size (in bytes) that is required for the given tensor reduction.
+ * \retval CUTENSOR_STATUS_SUCCESS The operation completed successfully.
+ * \retval CUTENSOR_STATUS_NOT_INITIALIZED if the handle is not initialized.
+ * \retval CUTENSOR_STATUS_INVALID_VALUE if some input data is invalid (this typically indicates an user error).
+ */
+cutensorStatus_t CUTENSOR_DEPRECATED(cutensorReductionGetWorkspaceSize) cutensorReductionGetWorkspace(const cutensorHandle_t* handle, 
+        const void* A, const cutensorTensorDescriptor_t* descA, const int32_t modeA[],
+        const void* C, const cutensorTensorDescriptor_t* descC, const int32_t modeC[],
+        const void* D, const cutensorTensorDescriptor_t* descD, const int32_t modeD[],
+        cutensorOperator_t opReduce, cutensorComputeType_t typeCompute, uint64_t *workspaceSize);
 #if defined(__cplusplus)
 }
 #endif /* __cplusplus */
